/* ADCS Example: Get Diagnostic Trouble Codes (DTCs) */

#include "get_dtc.h"
#include "../libnidiagcs-enums.h"
#include <nidiagcs.h>

#include <stdlib.h>
#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#define ERROR_STRING_LEN 1024

int printLabViewStatus(void);
void printAdcsStatus(int32_t status);
void readParameters(int argc, char* argv[]);
void Example_GetDTCs(void);
void printDTCs(TD4 DTCs[], int32_t DTCArrayLen);

int clearDTCs = 0; /* Variable to handle parameter passed to the example */


int main(int argc, char *argv[])
{
   printf("NI-ADCS Example: Get Diagnostic Trouble Codes (DTCs)\n");

   /* Make sure that LV runtime and dependencies were loaded successfully, abort otherwise */
   if (printLabViewStatus() != mgNoErr)
   {
      fprintf(stderr, "Couldn't load NI-ADCS library, make sure the product is installed correctly.\nAborting...\n");
      exit(EXIT_FAILURE);
   }

   /* Read parameters passed to the example */
   readParameters(argc, argv);

   /* Run the example */
   Example_GetDTCs();

   return EXIT_SUCCESS;
}


void Example_GetDTCs(void)
{
   /* CAN Settings */
   char CANInterface[] = "CAN1@nixnet";
   const uint64_t baudrate = 500000;
   const uint64_t FDBaudrate = 0;
   const uint32_t ioMode = ADCS_IOMODE_CAN_20;
   const uint16_t transportProtocol = ADCS_TP_ISOTP_NORMAL;
   const uint16_t maxDLC = 8;
   const uint32_t transmitID = 0x200;
   const uint32_t receiveID = 0x201;

   /* ADCS Definitions */
   const uint32_t GroupOfDTC = 0xFFFFFF;
   const uint8_t StatusMask = 0xFF;
   const int32_t Num_DTCs = 200;

   TD3 DTCDescriptor =
   {
      3,
      1,
      0,
      0
   }; /* Struct that describes the type of DTCs the ECU delivers */

   /* Data variables */
   int32_t status = SUCCESS;       /* Status return value */
   LVBoolean success = LVFALSE;    /* Service success return value */
   TD1 diagSessionRef;             /* Diagnostic Session Reference */
   uint8_t statusAvailMask;        /* Application-specific value returned for all DTCs */
   TD4 DTCs[Num_DTCs];             /* Buffer to store received DTCs */
   int32_t DTCArrayLen = Num_DTCs; /* On input: Size of DTCs buffer
                                      On output: Number of valid elements in the DTCs buffer */

   printf("Using %s at %lu baud (BRS: %lu baud)\n", CANInterface, baudrate, FDBaudrate);

   /* Open a Diagnostic Session on CAN */
   status = ndOpenDiagnosticOnCANFD(CANInterface, baudrate, FDBaudrate, ioMode, transportProtocol, maxDLC, transmitID, receiveID, &diagSessionRef);
   if (NO_ERROR(status))
   {
      /* Session has been created */

      /* Set the ECU Diagnostic Session */
      status = ndUDSDiagnosticSessionControl(&diagSessionRef, ADCS_UDS_SESSIONMODE_PROGRAMMING_SESSION, &success);

      /* Clear selected DTCs, if chosen */
      if (clearDTCs)
      {
         if (NO_ERROR(status))
         {
            status = ndUDSClearDiagnosticInformation(&diagSessionRef, GroupOfDTC, &DTCDescriptor, &success);
            printf("Status of clear selected DTCs: ");
            printf(NO_ERROR(status) ? "success\n" : "failure\n");
         }
      }

      /* Read out the DTCs */
      if (NO_ERROR(status))
      {
         status = ndUDSReportDTCByStatusMask(&diagSessionRef, StatusMask, &DTCDescriptor, &statusAvailMask, DTCs, &DTCArrayLen, &success);
      }

      /* Print DTCs */
      if (NO_ERROR(status))
      {
         printDTCs(DTCs, DTCArrayLen);
      }

      /* Finally, close the Diagnostic Session */
      ndCloseDiagnostic(&diagSessionRef);
   }

   printf("Example finished, final status: ");
   printAdcsStatus(status);
   printf("\n");
}


void readParameters(int argc, char* argv[])
{
   /* Supported parameters: -c to clear DTCs */
   for (int i=1; i<argc; i++)
   {
      if (argv[i][0] == '-')
      {
         switch (tolower(argv[i][1]))
         {
         case 'c':
            clearDTCs = 1;
            break;
         default:
            fprintf(stderr, "Parameter passed to get_dtc.c example is not supported.\nAborting...\n");
            exit(EXIT_FAILURE);
         }
      }
   }
}


void printDTCs (TD4 DTCs[], int32_t DTCArrayLen)
{
   printf("\tDTC(hex)\tStatus(hex)\n\t-------------------\n");
   for (int i=0; i<DTCArrayLen; i++)
   {
      printf("\t%X\t\t%02X\n", DTCs[i].DTC, DTCs[i].Status);
   }
}


MgErr printLabViewStatus(void)
{
   char errStr[ERROR_STRING_LEN] = "";
   const MgErr e = LVDLLStatus(errStr, ERROR_STRING_LEN, NULL);
   if (e != mgNoErr)
   {
      fprintf(stderr, "LabVIEW error %d, %s\n", e, errStr);
   }
   return e;
}


void printAdcsStatus(const int32_t status)
{
   char errStr[ERROR_STRING_LEN] = "";
   int errStrLen = ERROR_STRING_LEN; /* Size of return data buffer */
   ndStatusToString(status, errStr, &errStrLen);
   printf("%s", errStr);
}