/* ADCS Example: Read Ecu Identification */

#include "adcs_read_ecu_identification.h"
#include "../libnidiagcs-enums.h"
#include <nidiagcs.h>

#include <stdlib.h>
#include <stdint.h>
#include <stdio.h>
#include <string.h>

#define ERROR_STRING_LEN 1024

int printLabViewStatus(void);
void printAdcsStatus(int32_t status);
void Example_ReadEcuIdentification(void);


int main()
{
   /* Make sure that LV runtime and dependencies were loaded successfully, abort otherwise */
   if (printLabViewStatus() != mgNoErr)
   {
      fprintf(stderr, "Couldn't load NI-ADCS library, make sure the product is installed correctly.\nAborting...\n");
      exit(EXIT_FAILURE);
   }

   /* Run the example */
   Example_ReadEcuIdentification();

   return EXIT_SUCCESS;
}


void Example_ReadEcuIdentification(void)
{
   /* ADCS Definitions */
   const uint16_t DID_EcuIdentification = 0xF081;

   /* CAN Settings */
   char CANInterface[] = "CAN1@nixnet";
   const uint64_t baudrate = 500000;
   const uint64_t FDBaudrate = 0;
   const uint32_t ioMode = ADCS_IOMODE_CAN_20;
   const uint16_t transportProtocol = ADCS_TP_ISOTP_NORMAL;
   const uint16_t maxDLC = 8;
   const uint32_t transmitID = 0x200;
   const uint32_t receiveID = 0x201;

   /* Data variables */
   int32_t status = SUCCESS; /* Status return value */
   LVBoolean success = LVFALSE; /* Service success return value */
   TD1 diagSessionRef; /* Diagnostic Session Reference */
   uint8_t dataout[128] = {0}; /* Return data buffer */
   int lenout = sizeof(dataout); /* Size of return data buffer */

   printf("NI-ADCS Example: Read ECU Identification\n");
   printf("Using %s at %lu baud (BRS: %lu baud)\n", CANInterface, baudrate, FDBaudrate);

   /* Open a Diagnostic Session on CAN */
   status = ndOpenDiagnosticOnCANFD(CANInterface, baudrate, FDBaudrate, ioMode, transportProtocol, maxDLC, transmitID, receiveID, &diagSessionRef);
   if (NO_ERROR(status))
   {
      /* Session has been created */

      /* Set the ECU Diagnostic Session */
      status = ndUDSDiagnosticSessionControl(&diagSessionRef, ADCS_UDS_SESSIONMODE_DEFAULT_SESSION, &success);

      /* Read out the ECU Identification */
      if (NO_ERROR(status))
      {
         status = ndUDSReadDataByIdentifier(&diagSessionRef, DID_EcuIdentification, dataout, &lenout, &success);
      }

      if (NO_ERROR(status))
      {
         printf("Received ECU Identification: '%.*s'\n", lenout, dataout);
      }

      /* Finally, close the Diagnostic Session */
      ndCloseDiagnostic(&diagSessionRef);
   }

   printf("Example finished, final status: ");
   printAdcsStatus(status);
   printf("\n");
}


MgErr printLabViewStatus(void)
{
   char errStr[ERROR_STRING_LEN] = "";
   const MgErr e = LVDLLStatus(errStr, ERROR_STRING_LEN, NULL);
   if (e != mgNoErr)
   {
      fprintf(stderr, "LabVIEW error %d, %s\n", e, errStr);
   }
   return e;
}


void printAdcsStatus(const int32_t status)
{
   char errStr[ERROR_STRING_LEN] = "";
   int errStrLen = ERROR_STRING_LEN; /* Size of return data buffer */
   ndStatusToString(status, errStr, &errStrLen);
   printf("%s", errStr);
}

