/* ADCS Example: Read/Write Data */

#include "read_write_data.h"
#include "../libnidiagcs-enums.h"
#include <nidiagcs.h>

#include <stdlib.h>
#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>

#define ERROR_STRING_LEN 1024

int printLabViewStatus(void);
void printAdcsStatus(int32_t status);
void readParameters(int argc, char* argv[]);
void Example_ReadWriteData(void);

/* Variables to handle parameters passed to the example */
int setPhysValue = 0;
double amplitude;
double frequency = 10.0; /* Frequency of the waveform is always set to 10.0 in this example */


int main(int argc, char *argv[])
{
   printf("NI-ADCS Example: Read/Write Data\n");

   /* Make sure that LV runtime and dependencies were loaded successfully, abort otherwise */
   if (printLabViewStatus() != mgNoErr)
   {
      fprintf(stderr, "Couldn't load NI-ADCS library, make sure the product is installed correctly.\nAborting...\n");
      exit(EXIT_FAILURE);
   }

   /* Read parameters passed to the example */
   readParameters(argc, argv);

   /* Run the example */
   Example_ReadWriteData();

   return EXIT_SUCCESS;
}


void Example_ReadWriteData(void)
{
   /* CAN Settings */
   char CANInterface[] = "CAN1@nixnet";
   const uint64_t baudrate = 500000;
   const uint64_t FDBaudrate = 0;
   const uint32_t ioMode = ADCS_IOMODE_CAN_20;
   const uint16_t transportProtocol = ADCS_TP_ISOTP_NORMAL;
   const uint16_t maxDLC = 8;
   const uint32_t transmitID = 0x200;
   const uint32_t receiveID = 0x201;

   /* ADCS Definitions */
   const uint16_t ReadID = 0x11;
   const uint16_t WriteID = 0x12;

   TD2 ReadTypeDescriptor =
   {
      0,
      2,
      0,
      1,
      0.001,
      0.0
   }; /* Type descriptor to convert binary data read from ECU to physical representation */
   
   TD2 FreqTypeDescriptor =
   {
      1,
      2,
      0,
      0,
      0.0001,
      0.0
   }; /* Type descriptor to convert frequency to binary representation before writing to ECU */

   TD2 AmplTypeDescriptor =
   {
      3,
      2,
      0,
      0,
      0.001,
      0.0
   }; /* Type descriptor to convert amplitude to binary representation before writing to ECU */

   const int32_t ReadDataLen = ReadTypeDescriptor.ByteLength;
   const int32_t WriteDataLen = AmplTypeDescriptor.ByteLength + FreqTypeDescriptor.ByteLength;

   /* Data variables */
   int32_t status = SUCCESS;        /* Status return value */
   LVBoolean success = LVFALSE;     /* Service success return value */
   TD1 diagSessionRef;              /* Diagnostic Session Reference */
   double physValue;                /* Physical value converted from binary representation */
   uint8_t readData[ReadDataLen];   /* Holds data read from the ECU */
   uint8_t writeData[WriteDataLen]; /* Holds data to be written to the ECU */

   printf("Using %s at %lu baud (BRS: %lu baud)\n", CANInterface, baudrate, FDBaudrate);

   /* Open a Diagnostic Session on CAN */
   status = ndOpenDiagnosticOnCANFD(CANInterface, baudrate, FDBaudrate, ioMode, transportProtocol, maxDLC, transmitID, receiveID, &diagSessionRef);
   if (NO_ERROR(status))
   {
      /* Session has been created */

      /* Set the ECU Diagnostic Session */
      status = ndUDSDiagnosticSessionControl(&diagSessionRef, ADCS_UDS_SESSIONMODE_PROGRAMMING_SESSION, &success);

      /* UDS write data */
      if (setPhysValue)
      {
         if (NO_ERROR(status))
         {
            /* Convert physical data to binary representation using type descriptor */
            int32_t writeLength = FreqTypeDescriptor.ByteLength;
            ndConvertFromPhys(&FreqTypeDescriptor, frequency, &writeData[0], &writeLength);
            writeLength = AmplTypeDescriptor.ByteLength;
            ndConvertFromPhys(&AmplTypeDescriptor, amplitude, &writeData[2], &writeLength);

            status = ndUDSWriteDataByIdentifier(&diagSessionRef, WriteID, writeData, WriteDataLen, &success);
            printf("Result of write data: ");
            printf(NO_ERROR(status) ? "success\n" : "failure\n");
            sleep (1);
         }
      }

      /* UDS read data */
      if (NO_ERROR(status))
      {
         printf ("\t  i\tData\n\t-------------------\n");
         for (int i=0; i<5; i++)
         {
            if (i>0)
            {
               sleep (1);
            }
            int32_t readLength = ReadDataLen;
            status = ndUDSReadDataByIdentifier(&diagSessionRef, ReadID, readData, &readLength, &success);
            if (NO_ERROR(status))
            {
               /* Convert binary representation to physical data using type descriptor */
               ndConvertToPhys(&ReadTypeDescriptor, readData, readLength, &physValue);
               printf("\t%3d\t%10.6f\n", i, physValue);
            }
         }
      }

      /* Finally, close the Diagnostic Session */
      ndCloseDiagnostic(&diagSessionRef);
   }

   printf("Example finished, final status: ");
   printAdcsStatus(status);
   printf("\n");
}


void readParameters(int argc, char* argv[])
{
   /* Supported values: -a for amplitude. E.g. -a2.0 sets amplitude to 2.0 */
   for (int i=1; i<argc; i++)
   {
      if (argv[i][0] == '-')
      {
         switch (tolower(argv[i][1]))
         {
         case 'a':
            setPhysValue = 1;
            amplitude = atof(&argv[i][2]);
            break;
         default:
            fprintf(stderr, "Parameter passed to read_write_data.c example is not supported.\nAborting...\n");
            exit(EXIT_FAILURE);
         }
      }
   }
}


MgErr printLabViewStatus(void)
{
   char errStr[ERROR_STRING_LEN] = "";
   const MgErr e = LVDLLStatus(errStr, ERROR_STRING_LEN, NULL);
   if (e != mgNoErr)
   {
      fprintf(stderr, "LabVIEW error %d, %s\n", e, errStr);
   }
   return e;
}


void printAdcsStatus(const int32_t status)
{
   char errStr[ERROR_STRING_LEN] = "";
   int errStrLen = ERROR_STRING_LEN; /* Size of return data buffer */
   ndStatusToString(status, errStr, &errStrLen);
   printf("%s", errStr);
}